@@ -1,7 +1,7 @@
-/* $Id: isosurf.c,v 1.4 1999/10/21 16:39:06 brianp Exp $ */
+/* $Id: isosurf.c,v 1.12 2001/06/04 15:34:31 keithw Exp $ */
 
 /*
- * Display an isosurface of 3-D wind speed volume.  
+ * Display an isosurface of 3-D wind speed volume.
  *
  * Command line options:
  *    -info      print GL implementation information
@@ -25,101 +25,92 @@
  * Other options are available via the popup menu.
  */
 
-/*
- * $Log: isosurf.c,v $
- * Revision 1.4  1999/10/21 16:39:06  brianp
- * added -info command line option
- *
- * Revision 1.3  1999/09/08 22:14:31  brianp
- * minor changes. always call compactify_arrays()
- *
- * Revision 1.2  1999/09/03 14:56:40  keithw
- * Fog, displaylist and zoom operations
- *
- * Revision 3.4  1999/04/24 01:10:47  keithw
- * clip planes, materials
- *
- * Revision 3.3  1999/03/31 19:42:14  keithw
- * support for cva
- *
- * Revision 3.1  1998/11/01 20:30:20  brianp
- * added benchmark feature (b key)
- *
- * Revision 3.0  1998/02/14 18:42:29  brianp
- * initial rev
- *
- */
-
-
-
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
 #include <string.h>
 #include <math.h>
+#define GL_GLEXT_LEGACY
 #include "GL/glut.h"
 
-#include "../util/readtex.c"   /* I know, this is a hack.  KW: me too. */
+#include "readtex.c"   /* I know, this is a hack.  KW: me too. */
 #define TEXTURE_FILE "../images/reflect.rgb"
 
-#define LIT                 0x1
-#define UNLIT               0x2
-#define TEXTURE             0x4
-#define NO_TEXTURE          0x8
-#define REFLECT             0x10
-#define NO_REFLECT          0x20
-#define POINT_FILTER        0x40
-#define LINEAR_FILTER       0x80
-#define GLVERTEX           0x100
-#define DRAW_ARRAYS         0x200 /* or draw_elts, if compiled */
-#define ARRAY_ELT           0x400
-#define COMPILED            0x800
-#define IMMEDIATE           0x1000
-#define SHADE_SMOOTH        0x2000
-#define SHADE_FLAT          0x4000
-#define TRIANGLES           0x8000
-#define STRIPS              0x10000
-#define USER_CLIP           0x20000
-#define NO_USER_CLIP        0x40000
-#define MATERIALS           0x80000
-#define NO_MATERIALS        0x100000
-#define FOG                 0x200000
-#define NO_FOG              0x400000
-#define QUIT                0x800000
-#define DISPLAYLIST         0x1000000
+#define LIT		0x1
+#define UNLIT		0x2
+#define REFLECT		0x10
+#define POINT_FILTER	0x40
+#define LINEAR_FILTER	0x80
+#define GLVERTEX	0x100
+#define DRAW_ELTS	0x4 
+#define DRAW_ARRAYS	0x200 
+#define ARRAY_ELT	0x400
+#define LOCKED	        0x800
+#define UNLOCKED	0x8 
+#define IMMEDIATE	0x1000
+#define DISPLAYLIST	0x2000000
+#define SHADE_SMOOTH	0x2000
+#define SHADE_FLAT	0x4000
+#define TRIANGLES	0x8000
+#define STRIPS		0x10000
+#define POINTS		0x20000
+#define USER_CLIP	0x40000
+#define NO_USER_CLIP	0x80000
+#define MATERIALS	0x100000
+#define NO_MATERIALS	0x200000
+#define FOG		0x400000
+#define NO_FOG		0x800000
+#define QUIT		0x1000000
+#define GLINFO		0x4000000
+#define STIPPLE		0x8000000
+#define NO_STIPPLE	0x20
 
-#define LIGHT_MASK  (LIT|UNLIT)
-#define TEXTURE_MASK (TEXTURE|NO_TEXTURE)
-#define REFLECT_MASK (REFLECT|NO_REFLECT)
-#define FILTER_MASK (POINT_FILTER|LINEAR_FILTER)
-#define RENDER_STYLE_MASK (GLVERTEX|DRAW_ARRAYS|ARRAY_ELT)
-#define COMPILED_MASK (COMPILED|IMMEDIATE|DISPLAYLIST)
-#define MATERIAL_MASK (MATERIALS|NO_MATERIALS)
-#define PRIMITIVE_MASK (TRIANGLES|STRIPS)
-#define CLIP_MASK (USER_CLIP|NO_USER_CLIP)
-#define SHADE_MASK (SHADE_SMOOTH|SHADE_FLAT)
-#define FOG_MASK (FOG|NO_FOG)
+#define LIGHT_MASK		(LIT|UNLIT|REFLECT)
+#define FILTER_MASK		(POINT_FILTER|LINEAR_FILTER)
+#define RENDER_STYLE_MASK	(GLVERTEX|DRAW_ARRAYS|DRAW_ELTS|ARRAY_ELT)
+#define DLIST_MASK		(IMMEDIATE|DISPLAYLIST)
+#define LOCK_MASK		(LOCKED|UNLOCKED)
+#define MATERIAL_MASK		(MATERIALS|NO_MATERIALS)
+#define PRIMITIVE_MASK		(TRIANGLES|STRIPS|POINTS)
+#define CLIP_MASK		(USER_CLIP|NO_USER_CLIP)
+#define SHADE_MASK		(SHADE_SMOOTH|SHADE_FLAT)
+#define FOG_MASK		(FOG|NO_FOG)
+#define STIPPLE_MASK		(STIPPLE|NO_STIPPLE)
 
 #define MAXVERTS 10000
+static GLuint maxverts = MAXVERTS;
 static float data[MAXVERTS][6];
 static float compressed_data[MAXVERTS][6];
+static float expanded_data[MAXVERTS*3][6];
 static GLuint indices[MAXVERTS];
 static GLuint tri_indices[MAXVERTS*3];
+static GLuint strip_indices[MAXVERTS];
 static GLfloat col[100][4];
 static GLint numverts, num_tri_verts, numuniq;
 
 static GLfloat xrot;
 static GLfloat yrot;
-static GLfloat dist = -6;
+static GLfloat dist;
 static GLint state, allowed = ~0;
 static GLboolean doubleBuffer = GL_TRUE;
-static GLdouble plane[4] = {1.0, 0.0, -1.0, 0.0};
-static GLuint surf1;
+static GLdouble plane[4];
+static GLuint surf1, dlist_state;
 
 static GLboolean PrintInfo = GL_FALSE;
 
-/* forward decl */
-int BuildList( int mode );
+
+static GLubyte halftone[] = {
+   0xAA, 0xAA, 0xAA, 0xAA, 0x55, 0x55, 0x55, 0x55, 0xAA, 0xAA, 0xAA, 0xAA,
+   0x55, 0x55, 0x55, 0x55, 0xAA, 0xAA, 0xAA, 0xAA, 0x55, 0x55, 0x55, 0x55,
+   0xAA, 0xAA, 0xAA, 0xAA, 0x55, 0x55, 0x55, 0x55, 0xAA, 0xAA, 0xAA, 0xAA,
+   0x55, 0x55, 0x55, 0x55, 0xAA, 0xAA, 0xAA, 0xAA, 0x55, 0x55, 0x55, 0x55,
+   0xAA, 0xAA, 0xAA, 0xAA, 0x55, 0x55, 0x55, 0x55, 0xAA, 0xAA, 0xAA, 0xAA,
+   0x55, 0x55, 0x55, 0x55, 0xAA, 0xAA, 0xAA, 0xAA, 0x55, 0x55, 0x55, 0x55,
+   0xAA, 0xAA, 0xAA, 0xAA, 0x55, 0x55, 0x55, 0x55, 0xAA, 0xAA, 0xAA, 0xAA,
+   0x55, 0x55, 0x55, 0x55, 0xAA, 0xAA, 0xAA, 0xAA, 0x55, 0x55, 0x55, 0x55,
+   0xAA, 0xAA, 0xAA, 0xAA, 0x55, 0x55, 0x55, 0x55, 0xAA, 0xAA, 0xAA, 0xAA,
+   0x55, 0x55, 0x55, 0x55, 0xAA, 0xAA, 0xAA, 0xAA, 0x55, 0x55, 0x55, 0x55,
+   0xAA, 0xAA, 0xAA, 0xAA, 0x55, 0x55, 0x55, 0x55};
 
 
 static void read_surface( char *filename )
@@ -133,7 +124,7 @@ static void read_surface( char *filename )
    }
 
    numverts = 0;
-   while (!feof(f) && numverts<MAXVERTS) {
+   while (!feof(f) && numverts<maxverts) {
       fscanf( f, "%f %f %f  %f %f %f",
 	      &data[numverts][0], &data[numverts][1], &data[numverts][2],
 	      &data[numverts][3], &data[numverts][4], &data[numverts][5] );
@@ -147,6 +138,31 @@ static void read_surface( char *filename )
 
 
 
+static void print_flags( const char *msg, GLuint flags ) 
+{
+   fprintf(stderr, 
+	   "%s (0x%x): %s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s\n",
+	   msg, flags,
+	   (flags & GLVERTEX) ? "glVertex, " : "",
+	   (flags & DRAW_ARRAYS) ? "glDrawArrays, " : "",
+	   (flags & DRAW_ELTS) ? "glDrawElements, " : "",
+	   (flags & ARRAY_ELT) ? "glArrayElement, " : "",
+	   (flags & LOCKED) ? "locked arrays, " : "",
+	   (flags & TRIANGLES) ? "GL_TRIANGLES, " : "",
+	   (flags & STRIPS) ? "GL_TRIANGLE_STRIP, " : "",
+	   (flags & POINTS) ? "GL_POINTS, " : "",
+	   (flags & DISPLAYLIST) ? "as a displaylist, " : "",
+	   (flags & LIT) ? "lit, " : "",
+	   (flags & UNLIT) ? "unlit, " : "",
+	   (flags & REFLECT) ? "reflect, " : "",
+	   (flags & SHADE_FLAT) ? "flat-shaded, " : "",
+	   (flags & USER_CLIP) ? "user_clip, " : "",
+	   (flags & MATERIALS) ? "materials, " : "",
+	   (flags & FOG) ? "fog, " : "",
+	   (flags & STIPPLE) ? "stipple, " : "");
+}
+
+
 
 struct data_idx {
    float *data;
@@ -188,7 +204,7 @@ int (*(compare[7]))( const void *a, const void *b ) =
 
 #define VEC_ELT(f, s, i)  (float *)(((char *)f) + s * i)
 
-static int sort_axis( int axis, 
+static int sort_axis( int axis,
 		      int vec_size,
 		      int vec_stride,
 		      struct data_idx *indices,
@@ -200,27 +216,27 @@ static int sort_axis( int axis,
 {
    int i;
 
-   if (finish-start > 2) 
+   if (finish-start > 2)
    {
       qsort( indices+start, finish-start, sizeof(*indices), compare[axis] );
-   } 
-   else if (indices[start].data[axis] > indices[start+1].data[axis]) 
+   }
+   else if (indices[start].data[axis] > indices[start+1].data[axis])
    {
       struct data_idx tmp = indices[start];
       indices[start] = indices[start+1];
       indices[start+1] = tmp;
    }
-	 
+
    if (axis == vec_size-1) {
       for (i = start ; i < finish ; ) {
 	 float max = indices[i].data[axis] + fudge;
 	 float *dest = VEC_ELT(out, vec_stride, uniq);
 	 int j;
-	
+
 	 for (j = 0 ; j < vec_size ; j++)
 	    dest[j] = indices[i].data[j];
 
-	 for ( ; i < finish && max >= indices[i].data[axis]; i++) 
+	 for ( ; i < finish && max >= indices[i].data[axis]; i++)
 	    indices[i].uniq_idx = uniq;
 
 	 uniq++;
@@ -235,7 +251,7 @@ static int sort_axis( int axis,
 	    int k;
 
 	    indices[i].uniq_idx = uniq;
-	
+
 	    for (k = 0 ; k < vec_size ; k++)
 	       dest[k] = indices[i].data[k];
 
@@ -252,7 +268,7 @@ static int sort_axis( int axis,
 }
 
 
-static void extract_indices1( const struct data_idx *in, unsigned int *out, 
+static void extract_indices1( const struct data_idx *in, unsigned int *out,
 			      int n )
 {
    int i;
@@ -274,13 +290,13 @@ static void compactify_arrays(void)
       ind[i].data = data[i];
    }
 
-   numuniq = sort_axis(0, 
-		       sizeof(compressed_data[0])/sizeof(float), 
+   numuniq = sort_axis(0,
+		       sizeof(compressed_data[0])/sizeof(float),
 		       sizeof(compressed_data[0]),
-		       ind, 
-		       0, 
-		       numverts, 
-		       (float *)compressed_data, 
+		       ind,
+		       0,
+		       numverts,
+		       (float *)compressed_data,
 		       0,
 		       1e-6);
 
@@ -290,6 +306,20 @@ static void compactify_arrays(void)
    free( ind );
 }
 
+static void expand_arrays(void) 
+{
+   int i;
+   int parity = 0;
+   for (i = 2 ; i < numverts ; i++, parity ^= 1) {
+      int v0 = i-2+parity;
+      int v1 = i-1-parity;
+      int v2 = i;
+      memcpy( expanded_data[(i-2)*3+0], data[v0], sizeof(data[0]) );
+      memcpy( expanded_data[(i-2)*3+1], data[v1], sizeof(data[0]) );
+      memcpy( expanded_data[(i-2)*3+2], data[v2], sizeof(data[0]) );
+   }
+}
+
 static float myrand( float max )
 {
    return max*rand()/(RAND_MAX+1.0);
@@ -305,7 +335,7 @@ static void make_tri_indices( void )
    for (j=2;j<numverts;j++,parity^=1) {
       if (parity) {
 	 *v++ = indices[j-1];
-	 *v++ = indices[j-2]; 
+	 *v++ = indices[j-2];
 	 *v++ = indices[j];
       } else {
 	 *v++ = indices[j-2];
@@ -313,7 +343,7 @@ static void make_tri_indices( void )
 	 *v++ = indices[j];
       }
    }
-   
+
    num_tri_verts = v - tri_indices;
    printf("num_tri_verts: %d\n", num_tri_verts);
 
@@ -323,6 +353,9 @@ static void make_tri_indices( void )
       col[j][1] = myrand(1);
       col[j][0] = myrand(1);
    }
+
+   for (i = 0; i < numverts ; i++)
+      strip_indices[i] = i;
 }
 
 #define MIN(x,y) (x < y) ? x : y
@@ -330,24 +363,33 @@ static void make_tri_indices( void )
 static void draw_surface( int with_state )
 {
    GLuint i, j;
+   
+   if (with_state & DISPLAYLIST) {
+      if ((with_state & (RENDER_STYLE_MASK|PRIMITIVE_MASK|MATERIAL_MASK)) != 
+	  dlist_state) {
+	 /* 
+	  */
+	 fprintf(stderr, "rebuilding displaylist\n");
 
-   switch (with_state & (COMPILED_MASK|RENDER_STYLE_MASK|PRIMITIVE_MASK)) {
+	 if (dlist_state)
+	    glDeleteLists( surf1, 1 );
+
+	 dlist_state = with_state & (RENDER_STYLE_MASK|PRIMITIVE_MASK|
+				     MATERIAL_MASK);
+	 surf1 = glGenLists(1);
+	 glNewList(surf1, GL_COMPILE);
+	 draw_surface( dlist_state );
+	 glEndList();
+      }
+
+      glCallList( surf1 );
+      return;
+   }
+
+   switch (with_state & (RENDER_STYLE_MASK|PRIMITIVE_MASK)) {
 #ifdef GL_EXT_vertex_array
 
-   case (COMPILED|DRAW_ARRAYS|STRIPS):
-      glDrawElements( GL_TRIANGLE_STRIP, numverts, GL_UNSIGNED_INT, indices );
-      break;
-
-
-   case (COMPILED|ARRAY_ELT|STRIPS):
-      glBegin( GL_TRIANGLE_STRIP );
-      for (i = 0 ; i < numverts ; i++) 
-	 glArrayElement( indices[i] );      
-      glEnd();
-      break;
-
-   case (COMPILED|DRAW_ARRAYS|TRIANGLES):
-   case (IMMEDIATE|DRAW_ARRAYS|TRIANGLES):
+   case (DRAW_ELTS|TRIANGLES):
       if (with_state & MATERIALS) {
 	 for (j = i = 0 ; i < num_tri_verts ; i += 600, j++) {
 	    GLuint nr = MIN(num_tri_verts-i, 600);
@@ -356,29 +398,16 @@ static void draw_surface( int with_state )
 	    glDrawElements( GL_TRIANGLES, nr, GL_UNSIGNED_INT, tri_indices+i );
 	 }
       } else {
-	 glDrawElements( GL_TRIANGLES, num_tri_verts, GL_UNSIGNED_INT, 
+	 glDrawElements( GL_TRIANGLES, num_tri_verts, GL_UNSIGNED_INT,
 			 tri_indices );
       }
-
       break;
 
-      /* Uses the original arrays (including duplicate elements):
-       */
-   case (IMMEDIATE|DRAW_ARRAYS|STRIPS):
-      glDrawArraysEXT( GL_TRIANGLE_STRIP, 0, numverts );
+   case (DRAW_ARRAYS|TRIANGLES):
+      glDrawArraysEXT( GL_TRIANGLES, 0, (numverts-2)*3 );
       break;
 
-      /* Uses the original arrays (including duplicate elements):
-       */
-   case (IMMEDIATE|ARRAY_ELT|STRIPS):
-      glBegin( GL_TRIANGLE_STRIP );
-      for (i = 0 ; i < numverts ; i++)
-	 glArrayElement( i );
-      glEnd();
-      break;
-
-   case (IMMEDIATE|ARRAY_ELT|TRIANGLES):
-   case (COMPILED|ARRAY_ELT|TRIANGLES):
+   case (ARRAY_ELT|TRIANGLES):
       if (with_state & MATERIALS) {
 	 for (j = i = 0 ; i < num_tri_verts ; i += 600, j++) {
 	    GLuint nr = MIN(num_tri_verts-i, 600);
@@ -394,12 +423,51 @@ static void draw_surface( int with_state )
 	 glBegin( GL_TRIANGLES );
 	 for (i = 0 ; i < num_tri_verts ; i++)
 	    glArrayElement( tri_indices[i] );
-	       
+
 	 glEnd();
-      }	 
+      }
       break;
 
-   case (IMMEDIATE|GLVERTEX|TRIANGLES):
+
+      /* Uses the original arrays (including duplicate elements):
+       */
+   case (DRAW_ARRAYS|STRIPS):
+      glDrawArraysEXT( GL_TRIANGLE_STRIP, 0, numverts );
+      break;
+   case (DRAW_ELTS|STRIPS):
+      glDrawElements( GL_TRIANGLE_STRIP, numverts,
+		      GL_UNSIGNED_INT, strip_indices );
+      break;
+
+      /* Uses the original arrays (including duplicate elements):
+       */
+   case (ARRAY_ELT|STRIPS):
+      glBegin( GL_TRIANGLE_STRIP );
+      for (i = 0 ; i < numverts ; i++)
+	 glArrayElement( i );
+      glEnd();
+      break;
+
+   case (DRAW_ARRAYS|POINTS):
+      glDrawArraysEXT( GL_POINTS, 0, numuniq );
+      break;
+   case (DRAW_ELTS|POINTS):
+      /* can use numuniq with strip_indices as strip_indices[i] == i.
+       */
+      glDrawElements( GL_POINTS, numuniq, 
+		      GL_UNSIGNED_INT, strip_indices );
+      break;
+   case (ARRAY_ELT|POINTS):
+      /* just emit each unique element once:
+       */
+      glBegin( GL_POINTS );
+      for (i = 0 ; i < numuniq ; i++)
+	 glArrayElement( i );
+      glEnd();
+      break;
+#endif
+
+   case (GLVERTEX|TRIANGLES):
       if (with_state & MATERIALS) {
 	 for (j = i = 0 ; i < num_tri_verts ; i += 600, j++) {
 	    GLuint nr = MIN(num_tri_verts-i, 600);
@@ -420,27 +488,35 @@ static void draw_surface( int with_state )
 	    glVertex3fv( &compressed_data[tri_indices[i]][0] );
 	 }
 	 glEnd();
-      }	 
+      }
       break;
 
-   case (DISPLAYLIST|GLVERTEX|STRIPS):
-      if (!surf1)
-	 surf1 = BuildList( GL_COMPILE_AND_EXECUTE );
-      else
-	 glCallList(surf1);
-      break;
-
-#endif
-
-      /* Uses the original arrays (including duplicate elements):
+   case (GLVERTEX|POINTS):
+      /* Renders all points, but not in strip order...  Shouldn't be a
+       * problem, but people may be confused as to why points are so
+       * much faster in this demo...  And why cva doesn't help them...
        */
-   default:
+      glBegin( GL_POINTS );
+      for ( i = 0 ; i < numuniq ; i++ ) {
+         glNormal3fv( &compressed_data[i][3] );
+         glVertex3fv( &compressed_data[i][0] );
+      }
+      glEnd();
+      break;
+
+   case (GLVERTEX|STRIPS):
       glBegin( GL_TRIANGLE_STRIP );
       for (i=0;i<numverts;i++) {
          glNormal3fv( &data[i][3] );
          glVertex3fv( &data[i][0] );
       }
       glEnd();
+      break;
+
+   default:
+      fprintf(stderr, "unimplemented mode %x...\n", 
+	      (with_state & (RENDER_STYLE_MASK|PRIMITIVE_MASK)));
+      break;
    }
 }
 
@@ -451,17 +527,9 @@ static void Display(void)
     glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
     draw_surface( state );
     glFlush();
-    if (doubleBuffer) glutSwapBuffers();    
+    if (doubleBuffer) glutSwapBuffers();
 }
 
-int BuildList( int mode )
-{
-   int rv = glGenLists(1);
-   glNewList(rv, mode );
-   draw_surface( IMMEDIATE|GLVERTEX|STRIPS );
-   glEndList();
-   return rv;
-}
 
 /* KW: only do this when necessary, so CVA can re-use results.
  */
@@ -523,25 +591,26 @@ static void InitMaterials(void)
     glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
     glLightfv(GL_LIGHT0, GL_POSITION, position0);
     glEnable(GL_LIGHT0);
-    
+
     glLightfv(GL_LIGHT1, GL_AMBIENT, ambient);
     glLightfv(GL_LIGHT1, GL_DIFFUSE, diffuse);
     glLightfv(GL_LIGHT1, GL_POSITION, position1);
     glEnable(GL_LIGHT1);
-    
+
     glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
     glLightModelfv(GL_LIGHT_MODEL_TWO_SIDE, lmodel_twoside);
 
     glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, front_mat_shininess);
     glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, front_mat_specular);
     glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, front_mat_diffuse);
+
+    glPolygonStipple (halftone);
 }
 
 
 
 #define UPDATE(o,n,mask) (o&=~mask, o|=n&mask)
-#define CHANGED(o,n,mask) ((n&mask) && \
-                           (n&mask) != (o&mask) ? UPDATE(o,n,mask) : 0)
+#define CHANGED(o,n,mask) ((n&mask) && (n&mask) != (o&mask) )
 
 static void ModeMenu(int m)
 {
@@ -549,10 +618,18 @@ static void ModeMenu(int m)
 
    if (!m) return;
 
-   if (m==QUIT) 
+   if (m==QUIT)
       exit(0);
 
+   if (m==GLINFO) {
+      printf("GL_VERSION: %s\n", (char *) glGetString(GL_VERSION));
+      printf("GL_EXTENSIONS: %s\n", (char *) glGetString(GL_EXTENSIONS));
+      printf("GL_RENDERER: %s\n", (char *) glGetString(GL_RENDERER));
+      return;
+   }
+
    if (CHANGED(state, m, FILTER_MASK)) {
+      UPDATE(state, m, FILTER_MASK);
       if (m & LINEAR_FILTER) {
 	 glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
 	 glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
@@ -563,13 +640,29 @@ static void ModeMenu(int m)
    }
 
    if (CHANGED(state, m, LIGHT_MASK)) {
-      if (m & LIT)
+      UPDATE(state, m, LIGHT_MASK);
+      if (m & LIT) {
 	 glEnable(GL_LIGHTING);
-      else
+	 glDisable(GL_TEXTURE_GEN_S);
+	 glDisable(GL_TEXTURE_GEN_T);
+	 glDisable(GL_TEXTURE_2D);
+      }
+      else if (m & UNLIT) {
 	 glDisable(GL_LIGHTING);
+	 glDisable(GL_TEXTURE_GEN_S);
+	 glDisable(GL_TEXTURE_GEN_T);
+	 glDisable(GL_TEXTURE_2D);
+      }
+      else if (m & REFLECT) {
+	 glDisable(GL_LIGHTING);
+	 glEnable(GL_TEXTURE_GEN_S);
+	 glEnable(GL_TEXTURE_GEN_T);
+	 glEnable(GL_TEXTURE_2D);
+      }
    }
 
    if (CHANGED(state, m, SHADE_MASK)) {
+      UPDATE(state, m, SHADE_MASK);
       if (m & SHADE_SMOOTH)
 	 glShadeModel(GL_SMOOTH);
       else
@@ -577,24 +670,8 @@ static void ModeMenu(int m)
    }
 
 
-   if (CHANGED(state, m, TEXTURE_MASK)) {
-      if (m & TEXTURE) 
-	 glEnable(GL_TEXTURE_2D);
-      else
-	 glDisable(GL_TEXTURE_2D);
-   }
-
-   if (CHANGED(state, m, REFLECT_MASK)) {
-      if (m & REFLECT) {
-	 glEnable(GL_TEXTURE_GEN_S);
-	 glEnable(GL_TEXTURE_GEN_T);
-      } else {
-	 glDisable(GL_TEXTURE_GEN_S);
-	 glDisable(GL_TEXTURE_GEN_T);
-      }
-   }
-
    if (CHANGED(state, m, CLIP_MASK)) {
+      UPDATE(state, m, CLIP_MASK);
       if (m & USER_CLIP) {
 	 glEnable(GL_CLIP_PLANE0);
       } else {
@@ -603,51 +680,121 @@ static void ModeMenu(int m)
    }
 
    if (CHANGED(state, m, FOG_MASK)) {
-      if (m & FOG) 
+      UPDATE(state, m, FOG_MASK);
+      if (m & FOG)
       {
 	 glEnable(GL_FOG);
-	 printf("FOG enable\n");
-      } 
-      else 
+      }
+      else
       {
 	 glDisable(GL_FOG);
-	 printf("FOG disable\n");
+      }
+   }
+
+   if (CHANGED(state, m, STIPPLE_MASK)) {
+      UPDATE(state, m, STIPPLE_MASK);
+      if (m & STIPPLE)
+      {
+	 glEnable(GL_POLYGON_STIPPLE);
+      }
+      else
+      {
+	 glDisable(GL_POLYGON_STIPPLE);
       }
    }
 
 #ifdef GL_EXT_vertex_array
-   if (CHANGED(state, m, (COMPILED_MASK|RENDER_STYLE_MASK|PRIMITIVE_MASK))) 
+   if (CHANGED(state, m, (LOCK_MASK|RENDER_STYLE_MASK|PRIMITIVE_MASK)))
    {
-      if ((m & (COMPILED_MASK|PRIMITIVE_MASK)) == (IMMEDIATE|STRIPS))
+      if (m & (PRIMITIVE_MASK)) {
+	 UPDATE(state, m, (PRIMITIVE_MASK));
+      }
+
+      if (m & (RENDER_STYLE_MASK)) {
+	 UPDATE(state, m, (RENDER_STYLE_MASK));
+      }
+
+      if (m & LOCK_MASK) {
+	 UPDATE(state, m, (LOCK_MASK));
+      }
+
+
+      print_flags("primitive", state & PRIMITIVE_MASK);
+      print_flags("render style", state & RENDER_STYLE_MASK);
+
+      if ((state & PRIMITIVE_MASK) != STRIPS &&
+	  ((state & RENDER_STYLE_MASK) == DRAW_ELTS ||
+	   (state & RENDER_STYLE_MASK) == ARRAY_ELT || 
+	   (state & PRIMITIVE_MASK) == POINTS))
       {
+	 fprintf(stderr, "enabling small arrays\n");
+	 /* Rendering any primitive with draw-element/array-element
+	  *  --> Can't do strips here as ordering has been lost in
+	  *  compaction process...
+	  */
+	 glVertexPointerEXT( 3, GL_FLOAT, sizeof(data[0]), numuniq,
+			     compressed_data );
+	 glNormalPointerEXT( GL_FLOAT, sizeof(data[0]), numuniq,
+			     &compressed_data[0][3]);
+#ifdef GL_EXT_compiled_vertex_array
+	 if (allowed & LOCKED) {
+	    if (state & LOCKED) {
+	       glLockArraysEXT( 0, numuniq );
+	    } else {
+	       glUnlockArraysEXT();
+	    }
+	 }
+#endif
+      }
+      else if ((state & PRIMITIVE_MASK) == TRIANGLES &&
+	       (state & RENDER_STYLE_MASK) == DRAW_ARRAYS) {
+	 fprintf(stderr, "enabling big arrays\n");
+	 /* Only get here for TRIANGLES and drawarrays
+	  */
+	 glVertexPointerEXT( 3, GL_FLOAT, sizeof(data[0]), (numverts-2) * 3,
+			     expanded_data );
+	 glNormalPointerEXT( GL_FLOAT, sizeof(data[0]), (numverts-2) * 3,
+			     &expanded_data[0][3]);
+
+#ifdef GL_EXT_compiled_vertex_array
+	 if (allowed & LOCKED) {
+	    if (state & LOCKED) {
+	       glLockArraysEXT( 0, (numverts-2)*3 );
+	    } else {
+	       glUnlockArraysEXT();
+	    }
+	 }
+#endif
+      }
+      else {
+	 fprintf(stderr, "enabling normal arrays\n");
 	 glVertexPointerEXT( 3, GL_FLOAT, sizeof(data[0]), numverts, data );
 	 glNormalPointerEXT( GL_FLOAT, sizeof(data[0]), numverts, &data[0][3]);
-      }
-      else 
-      {
-	 glVertexPointerEXT( 3, GL_FLOAT, sizeof(data[0]), numuniq, 
-			     compressed_data );
-	 glNormalPointerEXT( GL_FLOAT, sizeof(data[0]), numuniq, 
-			     &compressed_data[0][3]);
-      }
 #ifdef GL_EXT_compiled_vertex_array
-      if (m & COMPILED) {
-	 glLockArraysEXT( 0, numuniq );
-      } else {
-	 glUnlockArraysEXT();
-      }
+	 if (allowed & LOCKED) {
+	    if (state & LOCKED) {
+	       glLockArraysEXT( 0, numverts );
+	    } else {
+	       glUnlockArraysEXT();
+	    }
+	 }
 #endif
+      }
+
    }
 #endif
 
-   if (m & (RENDER_STYLE_MASK|PRIMITIVE_MASK)) {
-      UPDATE(state, m, (RENDER_STYLE_MASK|PRIMITIVE_MASK));
+
+   if (m & DLIST_MASK) {
+      UPDATE(state, m, DLIST_MASK);
    }
-   
+
    if (m & MATERIAL_MASK) {
       UPDATE(state, m, MATERIAL_MASK);
    }
 
+   print_flags("new flags", state);
+
    glutPostRedisplay();
 }
 
@@ -657,33 +804,37 @@ static void Init(int argc, char *argv[])
 {
    GLfloat fogColor[4] = {0.5,1.0,0.5,1.0};
 
-   glClearColor(0.0, 0.0, 0.0, 0.0);
+   xrot = 0;
+   yrot = 0;
+   dist = -6;
+   plane[0] = 1.0;
+   plane[1] = 0.0;
+   plane[2] = -1.0;
+   plane[3] = 0.0;
+
+   glClearColor(0.0, 0.0, 1.0, 0.0);
    glEnable( GL_DEPTH_TEST );
    glEnable( GL_VERTEX_ARRAY_EXT );
    glEnable( GL_NORMAL_ARRAY_EXT );
 
-   InitMaterials();
-
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glFrustum( -1.0, 1.0, -1.0, 1.0, 5, 25 );
 
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
-   glClipPlane(GL_CLIP_PLANE0, plane); 
+   glClipPlane(GL_CLIP_PLANE0, plane);
+
+   InitMaterials();
 
    set_matrix();
 
-   glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
+   glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_FASTEST);
 
    glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
    glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
 
-   if (!LoadRGBMipmaps(TEXTURE_FILE, GL_RGB)) {
-      printf("Error: couldn't load texture image\n");
-      exit(1);
-   }
 
    /* Green fog is easy to see */
    glFogi(GL_FOG_MODE,GL_EXP2);
@@ -691,21 +842,31 @@ static void Init(int argc, char *argv[])
    glFogf(GL_FOG_DENSITY,0.15);
    glHint(GL_FOG_HINT,GL_DONT_CARE);
 
+   {
+      static int firsttime = 1;
+      if (firsttime) {
+	 firsttime = 0;
+	 compactify_arrays();
+	 expand_arrays();
+	 make_tri_indices();
 
-   compactify_arrays();
-   make_tri_indices();
-
-   surf1 = BuildList( GL_COMPILE );
+	 if (!LoadRGBMipmaps(TEXTURE_FILE, GL_RGB)) {
+	    printf("Error: couldn't load texture image\n");
+	    exit(1);
+	 }
+      }
+   }
 
    ModeMenu(SHADE_SMOOTH|
 	    LIT|
-	    NO_TEXTURE|
-	    NO_REFLECT|
 	    POINT_FILTER|
-	    IMMEDIATE|
 	    NO_USER_CLIP|
 	    NO_MATERIALS|
 	    NO_FOG|
+	    NO_STIPPLE|
+	    IMMEDIATE|
+	    STRIPS|
+	    UNLOCKED|
 	    GLVERTEX);
 
    if (PrintInfo) {
@@ -738,8 +899,11 @@ static void Key( unsigned char key, int x, int y )
    case 's':
       ModeMenu((state ^ SHADE_MASK) & SHADE_MASK);
       break;
+   case 't':
+      ModeMenu((state ^ STIPPLE_MASK) & STIPPLE_MASK);
+      break;
    case 'l':
-      ModeMenu((state ^ LIGHT_MASK) & LIGHT_MASK);
+      ModeMenu((state ^ LIGHT_MASK) & (LIT|UNLIT));
       break;
    case 'm':
       ModeMenu((state ^ MATERIAL_MASK) & MATERIAL_MASK);
@@ -747,6 +911,12 @@ static void Key( unsigned char key, int x, int y )
    case 'c':
       ModeMenu((state ^ CLIP_MASK) & CLIP_MASK);
       break;
+   case 'v':
+      ModeMenu((LOCKED|IMMEDIATE|DRAW_ELTS|TRIANGLES) & allowed);
+      break;
+   case 'V':
+      ModeMenu(UNLOCKED|IMMEDIATE|GLVERTEX|STRIPS);
+      break;
    case 'b':
       Benchmark(5.0, 0);
       break;
@@ -781,7 +951,9 @@ static void Key( unsigned char key, int x, int y )
       set_matrix();
       glutPostRedisplay();
       break;
-
+   case ' ':
+      Init(0,0);
+      break;
    }
 }
 
@@ -827,6 +999,15 @@ static GLint Args(int argc, char **argv)
       else if (strcmp(argv[i], "-info") == 0) {
          PrintInfo = GL_TRUE;
       }
+      else if (strcmp(argv[i], "-10") == 0) {
+         maxverts = 10;
+      }
+      else if (strcmp(argv[i], "-100") == 0) {
+	 maxverts = 100;
+      }
+      else if (strcmp(argv[i], "-1000") == 0) {
+	 maxverts = 1000;
+      }
       else {
          printf("%s (Bad option).\n", argv[i]);
 	 return QUIT;
@@ -850,7 +1031,7 @@ int main(int argc, char **argv)
 
    glutInitWindowPosition(0, 0);
    glutInitWindowSize(400, 400);
-   
+
    type = GLUT_DEPTH;
    type |= GLUT_RGB;
    type |= (doubleBuffer) ? GLUT_DOUBLE : GLUT_SINGLE;
@@ -863,71 +1044,66 @@ int main(int argc, char **argv)
    /* Make sure server supports the vertex array extension */
    extensions = (char *) glGetString( GL_EXTENSIONS );
 
-   if (!strstr( extensions, "GL_EXT_vertex_array" )) 
+   if (!strstr( extensions, "GL_EXT_vertex_array" ))
    {
       printf("Vertex arrays not supported by this renderer\n");
-      allowed &= ~(COMPILED|DRAW_ARRAYS|ARRAY_ELT);
+      allowed &= ~(LOCKED|DRAW_ARRAYS|DRAW_ELTS|ARRAY_ELT);
    }
-   else if (!strstr( extensions, "GL_EXT_compiled_vertex_array" )) 
+   else if (!strstr( extensions, "GL_EXT_compiled_vertex_array" ))
    {
       printf("Compiled vertex arrays not supported by this renderer\n");
-      allowed &= ~COMPILED;
+      allowed &= ~LOCKED;
    }
 
    Init(argc, argv);
    ModeMenu(arg_mode);
-   
+
    glutCreateMenu(ModeMenu);
-   glutAddMenuEntry("Lit",                   LIT|NO_TEXTURE|NO_REFLECT);
-   glutAddMenuEntry("Unlit",                 UNLIT|NO_TEXTURE|NO_REFLECT);
-/*    glutAddMenuEntry("Textured", TEXTURE); */
-   glutAddMenuEntry("Reflect",               TEXTURE|REFLECT);
-   glutAddMenuEntry("", 0);   
+   glutAddMenuEntry("GL info",               GLINFO);
+   glutAddMenuEntry("", 0);
+   glutAddMenuEntry("Lit",                   LIT);
+   glutAddMenuEntry("Unlit",                 UNLIT);
+   glutAddMenuEntry("Reflect",               REFLECT);
+   glutAddMenuEntry("", 0);
    glutAddMenuEntry("Smooth",                SHADE_SMOOTH);
    glutAddMenuEntry("Flat",                  SHADE_FLAT);
-   glutAddMenuEntry("", 0);   
+   glutAddMenuEntry("", 0);
    glutAddMenuEntry("Fog",                   FOG);
    glutAddMenuEntry("No Fog",                NO_FOG);
-   glutAddMenuEntry("", 0);   
+   glutAddMenuEntry("", 0);
+   glutAddMenuEntry("Stipple",               STIPPLE);
+   glutAddMenuEntry("No Stipple",            NO_STIPPLE);
+   glutAddMenuEntry("", 0);
    glutAddMenuEntry("Point Filtered",        POINT_FILTER);
    glutAddMenuEntry("Linear Filtered",       LINEAR_FILTER);
-   glutAddMenuEntry("", 0);   
-   glutAddMenuEntry("glVertex (STRIPS)",    IMMEDIATE|GLVERTEX|STRIPS);
-   glutAddMenuEntry("glVertex (TRIANGLES)", IMMEDIATE|GLVERTEX|TRIANGLES);
-   glutAddMenuEntry("", 0);   
-   glutAddMenuEntry("glVertex display list (STRIPS)", 
-		    DISPLAYLIST|GLVERTEX|STRIPS);
-   glutAddMenuEntry("", 0);   
+   glutAddMenuEntry("", 0);
+   glutAddMenuEntry("GL_TRIANGLES",          TRIANGLES);
+   glutAddMenuEntry("GL_TRIANGLE_STRIPS",    STRIPS);
+   glutAddMenuEntry("GL_POINTS",             POINTS);
+   glutAddMenuEntry("", 0);
+   glutAddMenuEntry("Displaylist",           DISPLAYLIST);
+   glutAddMenuEntry("Immediate",             IMMEDIATE);
+   glutAddMenuEntry("", 0);
+   if (allowed & LOCKED) {
+      glutAddMenuEntry("Locked Arrays (CVA)", LOCKED);
+      glutAddMenuEntry("Unlocked Arrays",     UNLOCKED);
+      glutAddMenuEntry("", 0);
+   }
+   glutAddMenuEntry("glVertex",               GLVERTEX);
    if (allowed & DRAW_ARRAYS) {
-      glutAddMenuEntry("DrawArrays (STRIPS)",
-		       IMMEDIATE|DRAW_ARRAYS|STRIPS);
-      glutAddMenuEntry("ArrayElement (STRIPS)",  
-		       IMMEDIATE|ARRAY_ELT|STRIPS);
-      glutAddMenuEntry("DrawElements (TRIANGLES)", 
-		       IMMEDIATE|DRAW_ARRAYS|TRIANGLES);
-      glutAddMenuEntry("ArrayElement (TRIANGLES)", 
-		       IMMEDIATE|ARRAY_ELT|TRIANGLES);
-      glutAddMenuEntry("", 0);   
-
+      glutAddMenuEntry("glDrawElements",      DRAW_ELTS);
+      glutAddMenuEntry("glDrawArrays",	      DRAW_ARRAYS);
+      glutAddMenuEntry("glArrayElement",      ARRAY_ELT);
    }
-   if (allowed & COMPILED) {
-      glutAddMenuEntry("Compiled DrawElements (TRIANGLES)", 
-		       COMPILED|DRAW_ARRAYS|TRIANGLES);
-      glutAddMenuEntry("Compiled DrawElements (STRIPS)", 
-		       COMPILED|DRAW_ARRAYS|STRIPS);
-      glutAddMenuEntry("Compiled ArrayElement (TRIANGLES)", 
-		       COMPILED|ARRAY_ELT|TRIANGLES);   
-      glutAddMenuEntry("Compiled ArrayElement (STRIPS)", 
-		       COMPILED|ARRAY_ELT|STRIPS);   
-      glutAddMenuEntry("", 0);   
-   }
-   glutAddMenuEntry("Quit",                  QUIT);
+   glutAddMenuEntry("", 0);
+   glutAddMenuEntry("Quit",                   QUIT);
    glutAttachMenu(GLUT_RIGHT_BUTTON);
 
    glutReshapeFunc(Reshape);
    glutKeyboardFunc(Key);
    glutSpecialFunc(SpecialKey);
    glutDisplayFunc(Display);
+
    glutMainLoop();
    return 0;
 }
