@@ -1,4 +1,4 @@
-/* $Id: multiarb.c,v 1.6 2000/05/23 23:21:00 brianp Exp $ */
+/* $Id: multiarb.c,v 1.10 2001/06/20 19:12:30 brianp Exp $ */
 
 /*
  * GL_ARB_multitexture demo
@@ -12,6 +12,41 @@
 
 /*
  * $Log: multiarb.c,v $
+ * Revision 1.10  2001/06/20 19:12:30  brianp
+ * also print GL_MAX_TEXTURE_SIZE
+ *
+ * Revision 1.9  2001/06/13 14:33:16  brianp
+ * moved glTexEnvi calls to better logical locations
+ *
+ * Revision 1.8  2000/12/24 22:53:54  pesco
+ * * demos/Makefile.am (INCLUDES): Added -I$(top_srcdir)/util.
+ * * demos/Makefile.X11, demos/Makefile.BeOS-R4, demos/Makefile.cygnus:
+ * Essentially the same.
+ * Program files updated to include "readtex.c", not "../util/readtex.c".
+ * * demos/reflect.c: Likewise for "showbuffer.c".
+ *
+ *
+ * * Makefile.am (EXTRA_DIST): Added top-level regular files.
+ *
+ * * include/GL/Makefile.am (INC_X11): Added glxext.h.
+ *
+ *
+ * * src/GGI/include/ggi/mesa/Makefile.am (EXTRA_HEADERS): Include
+ * Mesa GGI headers in dist even if HAVE_GGI is not given.
+ *
+ * * configure.in: Look for GLUT and demo source dirs in $srcdir.
+ *
+ * * src/swrast/Makefile.am (libMesaSwrast_la_SOURCES): Set to *.[ch].
+ * More source list updates in various Makefile.am's.
+ *
+ * * Makefile.am (dist-hook): Remove CVS directory from distribution.
+ * (DIST_SUBDIRS): List all possible subdirs here.
+ * (SUBDIRS): Only list subdirs selected for build again.
+ * The above two applied to all subdir Makefile.am's also.
+ *
+ * Revision 1.7  2000/11/01 16:02:01  brianp
+ * print number of texture units
+ *
  * Revision 1.6  2000/05/23 23:21:00  brianp
  * set default window pos
  *
@@ -48,7 +83,7 @@
 #include <string.h>
 #include <GL/glut.h>
 
-#include "../util/readtex.c"   /* I know, this is a hack. */
+#include "readtex.c"   /* I know, this is a hack. */
 
 #define TEXTURE_1_FILE "../images/girl.rgb"
 #define TEXTURE_2_FILE "../images/reflect.rgb"
@@ -246,6 +281,7 @@ static void SpecialKey( int key, int x, int y )
 static void Init( int argc, char *argv[] )
 {
    GLuint texObj[2];
+   GLint units, size;
 
    const char *exten = (const char *) glGetString(GL_EXTENSIONS);
    if (!strstr(exten, "GL_ARB_multitexture")) {
@@ -253,6 +289,14 @@ static void Init( int argc, char *argv[] )
       exit(1);
    }
 
+   glGetIntegerv(GL_MAX_TEXTURE_UNITS_ARB, &units);
+   printf("%d texture units supported\n", units);
+
+   glGetIntegerv(GL_MAX_TEXTURE_SIZE, &size);
+   printf("%d x %d max texture size\n", size, size);
+
+   glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
+
    /* allocate two texture objects */
    glGenTextures(2, texObj);
 
@@ -266,11 +310,6 @@ static void Init( int argc, char *argv[] )
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
 #endif
-
-   glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
-
-   glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
-
    if (!LoadRGBMipmaps(TEXTURE_1_FILE, GL_RGB)) {
       printf("Error: couldn't load texture image\n");
       exit(1);
@@ -287,9 +326,6 @@ static void Init( int argc, char *argv[] )
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
 #endif
-
-   glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
-
    if (!LoadRGBMipmaps(TEXTURE_2_FILE, GL_RGB)) {
       printf("Error: couldn't load texture image\n");
       exit(1);
@@ -300,8 +336,10 @@ static void Init( int argc, char *argv[] )
 #ifdef GL_ARB_multitexture
    glActiveTextureARB(GL_TEXTURE0_ARB);
    glBindTexture(GL_TEXTURE_2D, texObj[0]);
+   glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
    glActiveTextureARB(GL_TEXTURE1_ARB);
    glBindTexture(GL_TEXTURE_2D, texObj[1]);
+   glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
 #endif
 
    glShadeModel(GL_FLAT);
